//////////////////////////////////////////////////////////////////////////////////
//	DataTypes.cs
//	Managed Wiimote Library
//	Written by Brian Peek (http://www.brianpeek.com/)
//  for MSDN's Coding4Fun (http://msdn.microsoft.com/coding4fun/)
//  Visit http://msdn.microsoft.com/coding4fun/hardware/article.aspx?articleid=1879033
//  for more information
//////////////////////////////////////////////////////////////////////////////////



namespace WiimoteLib
{
	/// <summary>
	/// Calibration information stored on the Nunchuk
	/// </summary>
	public struct NunchukCalibrationInfo
	{
		public byte X0, Y0, Z0;
		public byte XG, YG, ZG;
		public byte MinX, MidX, MaxX;
		public byte MinY, MidY, MaxY;
	}

	/// <summary>
	/// Calibration information stored on the Classic Controller
	/// </summary>
	public struct ClassicControllerCalibrationInfo
	{
		public byte MinXL, MidXL, MaxXL;
		public byte MinYL, MidYL, MaxYL;
		public byte MinXR, MidXR, MaxXR;
		public byte MinYR, MidYR, MaxYR;
		public byte MinTriggerL, MaxTriggerL;
		public byte MinTriggerR, MaxTriggerR;
	}

	/// <summary>
	/// Current state of the Nunchuk extension
	/// </summary>
	public struct NunchukState
	{
		public NunchukCalibrationInfo CalibrationInfo;
		public AccelState AccelState;
		public byte RawX, RawY;
		public float X, Y;
		public bool C;
		public bool Z;
	}

	/// <summary>
	/// Curernt button state of the Classic Controller
	/// </summary>
	public struct ClassicControllerButtonState
	{
		public bool A;
		public bool B;
		public bool Plus;
		public bool Home;
		public bool Minus;
		public bool Up;
		public bool Down;
		public bool Left;
		public bool Right;
		public bool X, Y;
		public bool ZL, ZR;
		public bool TriggerL, TriggerR;
	}

	/// <summary>
	/// Current state of the Classic Controller
	/// </summary>
	public struct ClassicControllerState
	{
		public ClassicControllerCalibrationInfo CalibrationInfo;
		public ClassicControllerButtonState ButtonState;
		public byte RawXL, RawYL;
		public byte RawXR, RawYR;
		public float XL, YL, XR, YR;
		public byte RawTriggerL, RawTriggerR;
		public float TriggerL, TriggerR;
	}

	/// <summary>
	/// Current state of the IR camera
	/// </summary>
	public struct IRState
	{
		public IRMode Mode;

		public int RawX1, RawX2;
		public int RawY1, RawY2;
		public int Size1, Size2;
		public bool Found1, Found2;

		public float X1, X2;
		public float Y1, Y2; 
	}

	/// <summary>
	/// Current state of the accelerometers
	/// </summary>
	public struct AccelState
	{
		public byte RawX, RawY, RawZ;
		public float X, Y, Z;
	}

	/// <summary>
	/// Calibration information stored on the Wiimote
	/// </summary>
	public struct CalibrationInfo
	{
		public byte X0, Y0, Z0;
		public byte XG, YG, ZG;
	}

	/// <summary>
	/// Current button state
	/// </summary>
	public struct ButtonState
	{
		public bool A;
		public bool B;
		public bool Plus;
		public bool Home;
		public bool Minus;
		public bool One;
		public bool Two;
		public bool Up;
		public bool Down;
		public bool Left;
		public bool Right;
	}

	/// <summary>
	/// The extension plugged into the Wiimote
	/// </summary>
	public enum ExtensionType : byte
	{
		None				= 0x00,
		Nunchuk				= 0xfe,
		ClassicController	= 0xfd,
	};

	/// <summary>
	/// The mode of data reported for the IR sensor
	/// </summary>
	public enum IRMode : byte
	{
		Off			= 0x00,
		Basic		= 0x01,	// 10 bytes
		Extended	= 0x03,	// 12 bytes
		Full		= 0x05,	// 16 bytes * 2 (format unknown)
	};
}
